package com.supervision.ai;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.json.JSONUtil;
import com.supervision.ai.dto.ChatReqDTO;
import com.supervision.ai.dto.ChatResDTO;
import com.supervision.ai.dto.MessageDTO;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import lombok.extern.slf4j.Slf4j;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
public class AiUtil {

    private static final String API_KEY = "789c02fa1000afd951145efff01e0461.tGLSHnIsiEEiWNNN";

    private static final String TOKEN;

    static {
        try {
            String[] split = API_KEY.split("\\.");
            String id = split[0];
            String secret = split[1];

            long nowMillis = System.currentTimeMillis();
            long exp = nowMillis + (24 * 60 * 60 * 1000);

            Map<String, Object> header = new HashMap<>();
            header.put("alg", "HS256");
            header.put("sign_type", "SIGN");

            Map<String, Object> payload = new HashMap<>();
            payload.put("api_key", id);
            payload.put("exp", exp);
            payload.put("timestamp", nowMillis);

            TOKEN = Jwts.builder()
                    .setHeader(header)
                    .setClaims(payload)
                    .setExpiration(new Date(exp))
                    .signWith(SignatureAlgorithm.HS256, secret.getBytes())
                    .compact();

        } catch (Exception e) {
            throw new RuntimeException("Invalid apikey", e);
        }
    }

    public static String chat(String message) {
        ChatReqDTO chatReqDTO = new ChatReqDTO();
        chatReqDTO.setModel("glm-3-turbo");
        MessageDTO messageDTO = new MessageDTO();
        messageDTO.setRole("user");
        messageDTO.setContent(message);
        chatReqDTO.setMessages(CollUtil.newArrayList(messageDTO));
        HttpResponse response = HttpRequest.post("https://open.bigmodel.cn/api/paas/v4/chat/completions")
                .body(JSONUtil.toJsonStr(chatReqDTO)).header("Authorization", TOKEN).execute();
        String body = response.body();
        ChatResDTO bean = JSONUtil.toBean(body, ChatResDTO.class);
        return bean.getChoices().stream().findFirst().orElseThrow(() -> new RuntimeException("获取政策文件失败")).getMessage().getContent();
    }

    public static String chatByMessage(List<MessageDTO> messageList) {
        ChatReqDTO chatReqDTO = new ChatReqDTO();
        chatReqDTO.setModel("glm-3-turbo");
        chatReqDTO.setMessages(messageList);
        HttpResponse response = HttpRequest.post("https://open.bigmodel.cn/api/paas/v4/chat/completions")
                .body(JSONUtil.toJsonStr(chatReqDTO)).header("Authorization", TOKEN).execute();
        String body = response.body();
        ChatResDTO bean = JSONUtil.toBean(body, ChatResDTO.class);
        return bean.getChoices().stream().findFirst().orElseThrow(() -> new RuntimeException("获取政策文件失败")).getMessage().getContent();
    }

}
